#ifndef __XLIC_LICENSING__
#define __XLIC_LICENSING__

#ifndef XLICD_SOCKETFILE
#define XLICD_SOCKETFILE "/run/xlicd/xlicd.sock"
#endif

#ifdef __cplusplus

#include <cstddef>
#include <cstdint>
#include <ctime>

extern "C" {

#else // __cplusplus

#include <stddef.h>
#include <stdint.h>
#include <time.h>

#endif // __cplusplus

typedef enum {
	XLIC_STATE_UNKNOWN = 0,
	XLIC_STATE_INACTIVE = 1,
	XLIC_STATE_ACTIVE = 2,
	XLIC_STATE_GRACE = 3,
	XLIC_STATE_INVALID = 4,
} xlic_license_state;

typedef enum {
	XLIC_TYPE_UNKNOWN = 0,
	XLIC_TYPE_PHYSICAL = 1,
	XLIC_TYPE_VIRTUAL = 2,
} xlic_license_type;

typedef enum {
	XLIC_RESULT_SUCCESS = 0,
	XLIC_RESULT_GENERIC_ERROR = -1, // general error
	XLIC_RESULT_SERVER_ERROR = -2, // server communication error
	XLIC_RESULT_DAEMON_ERROR = -3, // daemon communication error
	XLIC_RESULT_REJECTED = -4, // request rejected by server
	XLIC_RESULT_INVALIDKEY = -5, // invalid license key
	XLIC_RESULT_NOLICENSE = -6, // nonexistent license key
	XLIC_RESULT_KEYEXISTS = -7, // license already exists
	XLIC_RESULT_UNKNOWN = -8,
	XLIC_RESULT_FOREIGN = -9, // foreign licenses
} xlic_result;

struct xlic_license_list {
	size_t count;
	struct xlic_license * licenses;
};

struct xlic_license {
	xlic_license_state state;
	xlic_license_type type;
	const char * key;
	const char * name;
	const char * description;
	time_t valid_after;
	size_t valid_for;
	size_t grace_days;
	size_t attribute_count;
	struct xlic_license_attribute * attributes;
};

struct xlic_license_attribute {
	const char * name;
	const char * value;
};

extern int xlic_fetch(const char * socket_filename);
extern int xlic_licenses(const char * socket_filename, struct xlic_license_list **);
extern void xlic_free(struct xlic_license_list * list);
extern int xlic_register_license(const char * socket_filename, const char * key);
extern int xlic_unregister_license(const char * socket_filename, const char * key);
extern const char * xlic_errstr(int code);
extern int xlic_update(const char * socket_filename);
extern int xlic_clear(const char * socket_filename);

#ifdef __cplusplus
}
#endif // __cplusplus

#endif // __XLIC_LICENSING__
