#ifndef __EXTERNAL_LICENSE_LIST__
#define __EXTERNAL_LICENSE_LIST__

#include <xlic/types.hpp>
#include <string>
#include <vector>

inline void convert(const xlic::license_state & from, xlic_license_state & to) {
	switch (from) {
		case xlic::license_state::inactive: {
			to = XLIC_STATE_INACTIVE;
			break;
		}
		case xlic::license_state::active: {
			to = XLIC_STATE_ACTIVE;
			break;
		}
		case xlic::license_state::grace: {
			to = XLIC_STATE_GRACE;
			break;
		}
		case xlic::license_state::invalid: {
			to = XLIC_STATE_INVALID;
			break;
		}
		default: {
			to = XLIC_STATE_UNKNOWN;
		}
	}
}

inline void convert(const xlic::license_type & from, xlic_license_type & to) {
	switch (from) {
		case xlic::license_type::physical_license: {
			to = XLIC_TYPE_PHYSICAL;
			break;
		}
		case xlic::license_type::virtual_license: {
			to = XLIC_TYPE_VIRTUAL;
			break;
		}
		default: {
			to = XLIC_TYPE_UNKNOWN;
			break;
		}
	}
}

struct external_details {
	std::string license_key;
	std::vector<xlic_license_attribute> attributes;
};

struct external_license_list : public xlic_license_list {
	inline external_license_list(xlic::license_list license_list)
	: m_license_list(std::move(license_list))
	{
		count = m_license_list.count();
		m_licenses.resize(count);
		m_details.resize(count);
		licenses = m_licenses.data();
		size_t i = 0;
		for (const auto & pair : m_license_list) {
			xlic_license & license = m_licenses[i];
			external_details & details = m_details[i++];
			convert(pair.second.state, license.state);
			convert(pair.second.type, license.type);
			details.license_key = to_string(pair.second.license_key);
			license.key = details.license_key.c_str();
			license.name = pair.second.name.c_str();
			license.description = pair.second.description.c_str();
			license.valid_for = pair.second.valid_for;
			license.grace_days = pair.second.grace_days;
			license.valid_after = pair.second.valid_after;
			license.attribute_count = pair.second.attributes.count();
			license.attributes = nullptr;
			if (license.attribute_count > 0) {
				details.attributes.resize(license.attribute_count);
				license.attributes = details.attributes.data();
				size_t j = 0;
				for (const auto & attribute : pair.second.attributes) {
					xlic_license_attribute & attr = details.attributes[j++];
					attr.name = attribute.first.c_str();
					attr.value = attribute.second.c_str();
				}
			}
		}
	}

	xlic::license_list m_license_list;
	std::vector<external_details> m_details;
	std::vector<xlic_license> m_licenses;
};

#endif
