#ifndef __XLIC_SERVER__
#define __XLIC_SERVER__

#include <xlic/types.hpp>
#include <xtl/exception.hpp>
#include <xtl/unix_acceptor.hpp>
#include <xtl/mutex.hpp>
#include <xtl/thread.hpp>
#include <string>
#include <unordered_map>
#include <memory>

namespace xlic {

class server_exception : public xtl::exception {
public:
	inline server_exception(const char * file, int line, const char * reason)
	: exception(file, line, "%s", reason) {}
};

class worker;

class server {
public:
	server(const std::string & filename);
	server(xlic::server && other) noexcept;
	server & operator=(xlic::server && other) noexcept;
	~server() noexcept;

	bool foreign_license_file() const noexcept;
	bool running() const noexcept;
	xtl::unix_acceptor & socket() noexcept;
	const xlic::mac_address_list & mac_addresses() const noexcept;
	xlic::serial_number serial_number();
	xtl::rsa_signature signature(const xlic::license_key & license_key) const;
	xlic::license_list licenses();
	void stop();
	void add(const xlic::serial_number & serial_number, const xtl::rsa_public_key & public_key, const xlic::license & license);
	bool exists(const xlic::license_key & license_key);
	void remove_license(const xlic::license_key & license_key);
	void update(const xlic::serial_number & serial_number, const xtl::rsa_public_key & public_key, const xlic::license_list & licenses, bool replace = false);
	void update_licenses();

private:
	bool m_have_dongle, m_foreign_license_file;
	time_t m_last_update;
	xtl::rsa_public_key m_public_key;
	xlic::serial_number m_serial_number, m_foreign_serial_number;
	xlic::mac_address_list m_mac_addresses, m_foreign_mac_addresses;
	xlic::license_list m_licenses;
	mutable xtl::mutex m_mutex;
	bool m_running;
	xtl::unix_acceptor m_socket;
	std::unordered_map<xlic::worker *, std::unique_ptr<xlic::worker>> m_workers;
	xtl::thread m_updater;

	bool load_licenses();
	void save_licenses();
};

inline bool server::foreign_license_file() const noexcept {
	return m_foreign_license_file;
}

inline bool server::running() const noexcept {
	return m_running;
}

inline xtl::unix_acceptor & server::socket() noexcept {
	return m_socket;
}

inline const xlic::mac_address_list & server::mac_addresses() const noexcept {
	return m_mac_addresses;
}

} // namespace xlic

#endif
