#ifndef __XLIC_TASK__
#define __XLIC_TASK__

#include <xlic/types.hpp>
#include <xtl/ostream.hpp>
#include <xtl/shims.hpp>
#include <memory>
#include <utility>

namespace xlic {

class session;
class worker;

class task {
public:
	task(xlic::session * session, xlic::worker & worker, xlic::request_id request_id) noexcept;
	inline virtual ~task() noexcept = default;

	void abandon() noexcept;
	bool abandoned() const noexcept;
	xlic::session * session() const noexcept;
	virtual void write_result(xtl::ostream & stream) = 0;

protected:
	xlic::session * m_session;
	xlic::worker & m_worker;
	xlic::request_id m_request_id;
	bool m_abandoned;
};

inline task::task(xlic::session * session, xlic::worker & worker, xlic::request_id request_id) noexcept
: m_session(session)
, m_worker(worker)
, m_request_id(request_id)
, m_abandoned(false)
{}

inline void task::abandon() noexcept {
	m_abandoned = true;
}

inline xlic::session * task::session() const noexcept {
	return m_session;
}

template<typename T, typename... argument_types>
inline std::unique_ptr<T> make_task(argument_types &&... arguments) {
	return std::make_unique<T>(std::forward<argument_types>(arguments)...);
}

inline bool task::abandoned() const noexcept {
	return m_abandoned;
}

} // namespace xlic

#endif
