#ifndef __XLIC_WORKER__
#define __XLIC_WORKER__

#include <xtl/fiber.hpp>
#include <xtl/pipe.hpp>
#include <xtl/unix_acceptor.hpp>
#include <xlic/server.hpp>
#include <xlic/types.hpp>
#include <unordered_map>
#include <memory>

namespace xlic {

class session;
class task;

class worker {
public:
	worker(xlic::server & server);
	worker(xlic::worker && other) noexcept;
	worker & operator=(xlic::worker && other) noexcept;
	~worker() noexcept;

	bool foreign_license_file() const noexcept;
	xlic::serial_number serial_number();
	const xlic::mac_address_list & mac_addresses() const noexcept;
	xtl::rsa_signature signature(const xlic::license_key & license_key) const noexcept;
	xlic::license_list licenses() const noexcept;
	void close(xlic::session * session);
	void add(std::unique_ptr<xlic::task> task);
	void add(const xlic::serial_number & serial_number, const xtl::rsa_public_key & public_key, const xlic::license & license);
	bool exists(const xlic::license_key & license_key);
	void remove_license(const xlic::license_key & license_key);
	void update(const xlic::serial_number & serial_number, const xtl::rsa_public_key & public_key, const xlic::license_list & licenses, bool replace = false);
	void stop() noexcept;
	void completed(xlic::task * task);
	void update_licenses();

private:
	bool m_active;
	xlic::server & m_server;
	std::unordered_map<xlic::session *, std::unique_ptr<xlic::session>> m_sessions;
	std::unordered_map<xlic::session *, std::unordered_map<xlic::task *, std::unique_ptr<xlic::task>>> m_tasks;
	xtl::pipe m_pipe;
	xtl::fiber m_acceptor;
	xtl::fiber m_completer;
};

inline bool worker::foreign_license_file() const noexcept {
	return m_server.foreign_license_file();
}

inline xlic::serial_number worker::serial_number() {
	return m_server.serial_number();
}

inline const xlic::mac_address_list & worker::mac_addresses() const noexcept {
	return m_server.mac_addresses();
}

inline xtl::rsa_signature worker::signature(const xlic::license_key & license_key) const noexcept {
	return m_server.signature(license_key);
}

inline xlic::license_list worker::licenses() const noexcept {
	return m_server.licenses();
}

inline void worker::add(const xlic::serial_number & serial_number, const xtl::rsa_public_key & public_key, const xlic::license & license) {
	m_server.add(serial_number, public_key, license);
}

inline bool worker::exists(const xlic::license_key & license_key) {
	return m_server.exists(license_key);
}

inline void worker::remove_license(const xlic::license_key & license_key) {
	m_server.remove_license(license_key);
}

inline void worker::update(const xlic::serial_number & serial_number, const xtl::rsa_public_key & public_key, const xlic::license_list & licenses, bool replace) {
	m_server.update(serial_number, public_key, licenses, replace);
}

inline void worker::stop() noexcept {
	m_active = false;
}

inline void worker::update_licenses() {
	m_server.update_licenses();
}

} // namespace xlic

#endif
