#include <xlic/clear_licenses.hpp>

#include <xtl/exception.hpp>
#include <xtl/https_stream.hpp>
#include <xlic/json.hpp>
#include <xlic/worker.hpp>

#if defined(DEBUG_CLEAR_LICENSES) || defined(DEBUG_ALL)
#define fdebug(fmt, ...) printf("clear_licenses[%d]: " fmt "\n", __LINE__, __VA_ARGS__);
#define debug(msg) printf("clear_licenses[%d]: " msg "\n", __LINE__);
#else
#define fdebug(fmt, ...)
#define debug(msg)
#endif

namespace xlic {

clear_licenses::clear_licenses(
	xlic::session * session,
	xlic::worker & worker,
	xlic::request_id request_id
)
: task(session, worker, request_id)
, m_result_code(xlic::result_code::unknown)
, m_thread([this] () {
	const auto licenses = m_worker.licenses();
	xtl::https_stream stream(XLIC_SERVER_HOSTNAME);
	for (const auto & pair : licenses) {
		try {
			stream.post("/unregister");
			stream.useragent(XLIC_USERAGENT);
			stream.content_type(XLIC_CONTENT_TYPE);
			stream << xlic::json_encode(pair.second.signature, pair.second.license_key);
			stream.flush();
		} catch (const std::exception & e) {
			fdebug("%s\n", e.what());
			m_result_code = xlic::result_code::generic_error;
		}
		m_worker.remove_license(pair.second.license_key);
	}
	m_worker.completed(this);
}) {}

void clear_licenses::write_result(xtl::ostream & stream) {
	stream << xlic::packet_size(xlic::packet_header_size);
	stream << xlic::packet_type::success;
	stream << m_request_id;
}

} // namespace xlic
