#include <xlic/unregister_license.hpp>

#include <xtl/exception.hpp>
#include <xtl/https_stream.hpp>
#include <xlic/json.hpp>
#include <xlic/worker.hpp>
#include <xlic/types.hpp>

#if defined(DEBUG_UNREGISTER_LICENSE) || defined(DEBUG_ALL)
#define fdebug(fmt, ...) printf("unregister_license[%d]: " fmt "\n", __LINE__, __VA_ARGS__);
#define debug(msg) printf("unregister_license[%d]: " msg "\n", __LINE__);
#else
#define fdebug(fmt, ...)
#define debug(msg)
#endif

namespace xlic {

unregister_license::unregister_license(
	xlic::session * session,
	xlic::worker & worker,
	xlic::request_id request_id,
	const xlic::license_key & license_key
)
: task(session, worker, request_id)
, m_result_code(xlic::result_code::unknown)
, m_thread([this, license_key] () {
	try {
		const auto signature = m_worker.signature(license_key);
		xtl::https_stream stream(XLIC_SERVER_HOSTNAME);
		stream.post("/unregister");
		stream.useragent(XLIC_USERAGENT);
		stream.content_type(XLIC_CONTENT_TYPE);
		stream << xlic::json_encode(signature, license_key);
		stream.flush();
		const xtl::json js(stream);
		if (js["message"].string() == "success") {
			m_result_code = xlic::result_code::success;
		} else {
			m_result_code = xlic::result_code::rejected;
		}
	} catch (const std::exception & e) {
		fdebug("%s\n", e.what());
		m_result_code = xlic::result_code::generic_error;
	}
	m_worker.remove_license(license_key);
	m_worker.completed(this);
}) {}

void unregister_license::write_result(xtl::ostream & stream) {
	if (m_result_code == xlic::result_code::success) {
		stream << xlic::packet_size(xlic::packet_header_size);
		stream << xlic::packet_type::success;
		stream << m_request_id;
	} else {
		stream << xlic::packet_size(xlic::packet_header_size + sizeof(m_result_code));
		stream << xlic::packet_type::error;
		stream << m_request_id;
		stream << m_result_code;
	}
}

} // namespace xlic
