#include <xlic/update_licenses.hpp>

#include <xtl/exception.hpp>
#include <xtl/https_stream.hpp>
#include <xlic/json.hpp>
#include <xlic/worker.hpp>

#if defined(DEBUG_FETCH_LICENSES) || defined(DEBUG_ALL)
#define fdebug(fmt, ...) printf("update_licenses[%d]: " fmt "\n", __LINE__, __VA_ARGS__);
#define debug(msg) printf("update_licenses[%d]: " msg "\n", __LINE__);
#else
#define fdebug(fmt, ...)
#define debug(msg)
#endif

namespace xlic {

update_licenses::update_licenses(
	xlic::session * session,
	xlic::worker & worker,
	xlic::request_id request_id
)
: task(session, worker, request_id)
, m_result_code(xlic::result_code::unknown)
{
	if (m_worker.foreign_license_file()) {
		m_result_code = xlic::result_code::foreign;
	} else {
		try {
			worker.update_licenses();
		} catch (const std::exception & e) {
			fdebug("%s\n", e.what());
			m_result_code = xlic::result_code::generic_error;
		}
	}
	m_worker.completed(this);
}

void update_licenses::write_result(xtl::ostream & stream) {
	if (m_result_code == xlic::result_code::success) {
		stream << xlic::packet_size(xlic::packet_header_size);
		stream << xlic::packet_type::success;
		stream << m_request_id;
	} else {
		stream << xlic::packet_size(xlic::packet_header_size + sizeof(m_result_code));
		stream << xlic::packet_type::error;
		stream << m_request_id;
		stream << m_result_code;
	}
}

} // namespace xlic
