#ifndef __XTL_EXCEPTION__
#define __XTL_EXCEPTION__

#include <string>
#include <cerrno>
#include <exception>

namespace xtl {

class exception : public std::exception {
public:
	exception(const char * file, int line, const char * format, ...);

	inline const char * what() const noexcept override { return m_what.c_str(); }

private:
	std::string m_what;
};

class invalid_argument : public xtl::exception {
public:
	inline invalid_argument(const char * file, int line)
	: exception(file, line, "%s", "Invalid argument") {}
};

class out_of_range : public xtl::exception {
public:
	inline out_of_range(const char * file, int line)
	: exception(file, line, "%s", "Out of range") {}
};

class timed_out : public xtl::exception {
public:
	inline timed_out(const char * file, int line)
	: exception(file, line, "%s", "Timed out") {}
};

class system_exception : public xtl::exception {
public:
	system_exception(const char * file, int line, int code = errno);
};

class logic_exception : public xtl::exception {
public:
	inline logic_exception(const char * file, int line, const std::string & reason)
	: exception(file, line, "%s", reason.c_str()) {}
};

class io_exception : public xtl::system_exception {
public:
	inline io_exception(const char * file, int line, int code = errno)
	: system_exception(file, line, code) {}
};

class access_denied : public xtl::io_exception {
public:
	inline access_denied(const char * file, int line, int code = EACCES)
	: io_exception(file, line, code) {}
};

class directory_not_found : public xtl::io_exception {
public:
	inline directory_not_found(const char * file, int line, int code = ENOTDIR)
	: io_exception(file, line, code) {}
};

class end_of_stream : public xtl::io_exception {
public:
	inline end_of_stream(const char * file, int line, int code = ENOTCONN)
	: io_exception(file, line, code) {}
};

class file_already_exists : public xtl::io_exception {
public:
	inline file_already_exists(const char * file, int line, int code = EEXIST)
	: io_exception(file, line, code) {}
};

class file_not_found : public xtl::io_exception {
public:
	inline file_not_found(const char * file, int line, int code = ENOENT)
	: io_exception(file, line, code) {}
};

class name_too_long : public xtl::io_exception {
public:
	inline name_too_long(const char * file, int line, int code = ENAMETOOLONG)
	: io_exception(file, line, code) {}
};

class quota_reached : public xtl::io_exception {
public:
	inline quota_reached(const char * file, int line, int code = EDQUOT)
	: io_exception(file, line, code) {}
};

class would_block : public xtl::io_exception {
public:
	inline would_block(const char * file, int line, int code = EWOULDBLOCK)
	: io_exception(file, line, code) {}
};

} // namespace xtl

#endif
