#ifndef __XTL_HTTPS_STREAM__
#define __XTL_HTTPS_STREAM__

#include <xtl/exception.hpp>
#include <xtl/iostream.hpp>
#include <cstdint>
#include <string>
#include <memory>

namespace xtl {

class https_stream_exception : public xtl::exception {
public:
	inline https_stream_exception(const char * file, int line, const char * reason)
	: exception(file, line, "%s", reason) {}
};

struct https_stream : public xtl::iostream, std::shared_ptr<class https_stream_impl> {
	explicit https_stream(const std::string & hostname, uint16_t port = 443);

	void get(const std::string & url);
	void post(const std::string & url);
	void put(const std::string & url);
	void set_timeout(size_t timeout_ms);
	void set_header(const std::string & name, const std::string & value);
	void unset_header(const std::string & name);
	void referer(const std::string & value);
	void useragent(const std::string & value);
	void content_type(const std::string & value);
	int code() const;

	size_t read(void * buffer, size_t amount) override;
	size_t write(const void * buffer, size_t amount) override;
	size_t flush() override;
};

} // namepspace xlic

#endif
