#ifndef __XTL_LOCK_GUARD__
#define __XTL_LOCK_GUARD__

namespace xtl {

template<typename T>
class lock_guard {
public:
	lock_guard(T & lock);
	lock_guard(const lock_guard &) = delete;
	lock_guard & operator=(const lock_guard &) = delete;
	~lock_guard() noexcept;

	void release() noexcept;

private:
	T * m_lock;
};

template<typename T>
inline lock_guard<T>::lock_guard(T & lock)
: m_lock(&lock) {
	m_lock->lock();
}

template<typename T>
inline lock_guard<T>::~lock_guard() noexcept {
	if (m_lock) {
		m_lock->unlock();
	}
}

template<typename T>
inline void lock_guard<T>::release() noexcept {
	if (m_lock) {
		m_lock->unlock();
		m_lock = nullptr;
	}
}

} // namespace xtl

#endif
