#ifndef __XTL_RSA__
#define __XTL_RSA__

#include <xtl/exception.hpp>
#include <xtl/verify_method.hpp>
#include <memory>

namespace xtl {

class rsa_exception : public xtl::exception {
public:
	inline rsa_exception(const char * file, int line, const char * reason)
	: exception(file, line, "%s", reason) {}
};

class rsa_signature {
public:
	inline void * value() noexcept { return m_value; }
	inline const void * value() const noexcept { return m_value; }
	inline size_t size() const noexcept { return sizeof(m_value); }

private:
	uint8_t m_value[64];
};

class rsa_public_key {
public:
	inline void * value() noexcept { return m_value; }
	inline const void * value() const noexcept { return m_value; }
	inline size_t size() const noexcept { return sizeof(m_value); }

private:
	uint8_t m_value[94];
};

class rsa_verify : public xtl::verify_method, std::shared_ptr<class rsa_verify_impl> {
public:
	rsa_verify(const xtl::rsa_public_key & public_key);

	verify_method & append(const void * buffer, size_t amount) override;
	bool verify(const xtl::rsa_signature & signature);
};

} // namespace xtl

#endif
