#ifndef __XTL_SHA256__
#define __XTL_SHA256__

#include <xtl/exception.hpp>
#include <xtl/hash_method.hpp>
#include <xtl/istream.hpp>
#include <xtl/ostream.hpp>
#include <cstdint>
#include <memory>

namespace xtl {

class sha256_exception : public xtl::exception {
public:
	inline sha256_exception(const char * file, int line, const char * reason)
	: exception(file, line, "%s", reason) {}
};

class sha256_digest {
public:
	inline void * value() noexcept { return m_value; }
	inline const void * value() const noexcept { return m_value; }
	inline size_t size() const noexcept { return sizeof(m_value); }

	bool operator==(const xtl::sha256_digest & other) const noexcept;
	bool operator!=(const xtl::sha256_digest & other) const noexcept;

	friend xtl::ostream & operator<<(xtl::ostream & stream, const xtl::sha256_digest & digest);
	friend xtl::istream & operator>>(xtl::istream & stream, xtl::sha256_digest & digest);

private:
	uint8_t m_value[32];
};

class sha256 : public xtl::hash_method, std::shared_ptr<class sha256_impl> {
public:
	sha256();

	hash_method & append(const void * buffer, size_t amount) override;
	xtl::sha256_digest digest() const;
};

} // namespace xtl

#endif
