#ifndef __XTL_SHIMS__
#define __XTL_SHIMS__

#ifndef __cpp_lib_exchange_function

#include <utility>

namespace std {

template<typename T, typename U = T>
inline T exchange(T & target, U && replacement) {
	T tmp = std::move(target);
	target = std::forward<U>(replacement);
	return tmp;
}

} // namespace std

#endif // __cpp_lib_exchange_function

#ifndef __cpp_lib_integer_sequence

#include <utility>

namespace std {

template<size_t... ints> struct index_tuple {};

template<typename left, typename right> struct index_tuple_pair;

template<size_t... left, size_t... right>
struct index_tuple_pair<std::index_tuple<left...>, std::index_tuple<right...>>
{
	using type = std::index_tuple<left..., (right + sizeof...(left))...>;
};

template<size_t N>
struct build_index_tuple : std::index_tuple_pair<typename std::build_index_tuple<N / 2>::type, typename std::build_index_tuple<N - N / 2>::type> {};

template<>
struct build_index_tuple<1> {
	typedef std::index_tuple<0> type;
};

template<>
struct build_index_tuple<0> {
	typedef std::index_tuple<> type;
};

template<typename T, T... ints>
struct integer_sequence {
	typedef T value_type;
	inline static constexpr size_t size() noexcept { return sizeof...(ints); }
};

template<typename T, T N, typename sequence = typename std::build_index_tuple<N>::type>
struct _make_integer_sequence;

template<typename T, T N, size_t... ints>
struct _make_integer_sequence<T, N, index_tuple<ints...>> {
	typedef std::integer_sequence<T, static_cast<T>(ints)...> type;
};

template<size_t... ints>
using index_sequence = std::integer_sequence<size_t, ints...>;

template<typename T, T N>
using make_integer_sequence = typename std::_make_integer_sequence<T, N>::type;

template<size_t N>
using make_index_sequence = std::make_integer_sequence<size_t, N>;

template<typename... T>
using index_sequence_for = std::make_index_sequence<sizeof...(T)>;

} // namespace std

#endif // __cpp_lib_integer_sequence

#ifndef __cpp_lib_make_unique

#include <memory>

namespace std {

template<typename T, typename... Ts>
inline std::unique_ptr<T> make_unique(Ts &&... arguments) {
	return std::unique_ptr<T>(new T(std::forward<Ts>(arguments)...));
}

template<typename T>
inline std::unique_ptr<T[]> make_unique(size_t N) {
	return std::unique_ptr<T[]>(new std::remove_extent<T>[N]());
}

} // namespace std

#endif // __cpp_lib_make_unique

#endif
