#ifndef __XTL_UTIL__
#define __XTL_UTIL__

#include <cstddef>
#include <string>
#include <utility>

namespace xtl {

	void dumphex(const void * ptr, size_t size, size_t line_length = 64);

	namespace details {

		inline constexpr size_t concat_size() noexcept { return 0; }

		template<typename... Ts> size_t concat_size(const std::string & string, Ts &&... strings);
		template<size_t N, typename... Ts> size_t concat_size(const char (& string)[N], Ts &&... strings);

		template<size_t N, typename... Ts>
		inline size_t concat_size(const char (&)[N], Ts &&... strings) {
			return (N - 1) + concat_size(std::forward<Ts>(strings)...);
		}

		template<typename... Ts>
		inline size_t concat_size(const std::string & string, Ts &&... strings) {
			return string.length() + concat_size(std::forward<Ts>(strings)...);
		}

		inline void concat_append(std::string &) noexcept {}

		template<typename T, typename... Ts>
		inline void concat_append(std::string & result, const T & string, Ts &&... strings) {
			result.append(string);
			concat_append(result, std::forward<Ts>(strings)...);
		}
	}

	template<typename... Ts>
	inline std::string concat(Ts &&... strings) {
		std::string result;
		result.reserve(details::concat_size(std::forward<Ts>(strings)...));
		details::concat_append(result, std::forward<Ts>(strings)...);
		return result;
	}

} // namespace xtl

#endif
