#include <xtl/fiber.hpp>
#include <xtl/scheduler.hpp>

#if defined(DEBUG_FIBER) || defined(DEBUG_ALL)
#define fdebug(fmt, ...) fprintf(stderr, "fiber[%d]: " fmt "\n", __LINE__, __VA_ARGS__);
#define debug(msg) fprintf(stderr, "fiber[%d]: " msg "\n", __LINE__);
#else
#define fdebug(fmt, ...)
#define debug(msg)
#endif

namespace xtl {

class fiber_impl {
public:
	inline fiber_impl(xtl::function<void()> & function, size_t stack_size)
	: m_handle(scheduler::create_fiber(std::move(function), stack_size))
	{}

	inline ~fiber_impl() noexcept {
		scheduler::destroy_fiber(m_handle);
	}

private:
	void * m_handle;
};

fiber::fiber(xtl::function<void()> function, size_t stack_size)
: shared_ptr(std::make_shared<xtl::fiber_impl>(function, stack_size))
{}

} // namespace xtl
