#include <xtl/fs.hpp>

#include <xtl/exception.hpp>

#include <cstdio>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

namespace xtl {

namespace fs {

size_t size(const std::string & filename) {
		struct stat buffer;
		if (::stat(filename.c_str(), &buffer) != 0) {
			throw xtl::system_exception(__FILE__, __LINE__);
		}
		return buffer.st_size;
}

void chmod(const std::string & filename, int mode) {
	if (::chmod(filename.c_str(), mode) != 0) {
		throw xtl::system_exception(__FILE__, __LINE__);
	}
}

void rename(const std::string & from, const std::string & to) {
	::rename(from.c_str(), to.c_str());
}

void truncate(const std::string & filename, size_t size) {
	for (;;) {
		if (::truncate(filename.c_str(), size) != 0) {
			throw xtl::system_exception(__FILE__, __LINE__);
		}
		return;
	}
}

void remove(const std::string & filename) {
	if (::unlink(filename.c_str()) != 0) {
		if (errno != ENOENT) {
			throw xtl::system_exception(__FILE__, __LINE__);
		}
	}
}

} // namespace fs

} // namespace xtl
