#include <xtl/os.hpp>

#include <xtl/exception.hpp>
#include <cerrno>
#include <cstdio>
#include <cstring>
#include <sys/resource.h>
#include <unistd.h>
#include <signal.h>

namespace xtl {

inline int disable_sigpipe() {
	::signal(SIGPIPE, SIG_IGN);
	return 0;
}

static const int dummy = disable_sigpipe();

bool mac_address::operator<(const xtl::mac_address & other) const noexcept {
	return (memcmp(m_value, other.m_value, sizeof(m_value)) < 0);
}

bool mac_address::operator==(const xtl::mac_address & other) const noexcept {
	return (memcmp(m_value, other.m_value, sizeof(m_value)) == 0);
}

std::string to_string(const xtl::mac_address & mac_address) {
	std::string result(17, '\0');
	const uint8_t * value = static_cast<const uint8_t *>(mac_address.value());
	snprintf(
		&result[0], 18,
		"%02x:%02x:%02x:%02x:%02x:%02x",
		value[0], value[1], value[2],
		value[3], value[4], value[5]
	);
	return result;
}

namespace os {

size_t processor_count() {
	const long result = ::sysconf(_SC_NPROCESSORS_ONLN);
	if (result < 0) {
		throw xtl::system_exception(__FILE__, __LINE__);
	}
	return result;
}

size_t page_size() {
	static size_t value = 0;
	if (value == 0) {
		const long result = ::sysconf(_SC_PAGESIZE);
		if (result < 0) {
			throw xtl::system_exception(__FILE__, __LINE__);
		}
		value = result;
	}
	return value;
}

size_t stack_size() {
	static size_t value = 0;
	if (value == 0) {
		struct rlimit rlim;
		if (getrlimit(RLIMIT_STACK, &rlim) != 0) {
			throw xtl::system_exception(__FILE__, __LINE__);
		}
		if (rlim.rlim_cur > 0) {
			value = rlim.rlim_cur;
		} else {
			value = 8*1024*1024; // 8 megs is the default on a test platform
		}
	}
	return value;
}

} // namespace os

} // namespace xtl
