#include <xtl/stack.hpp>

#include <xtl/exception.hpp>
#include <xtl/os.hpp>
#include <sys/mman.h>
#include <cstdint>
#include <cerrno>
#include <cstdio>

namespace xtl {

stack::stack(size_t stack_size)
: m_size(0)
, m_base(nullptr)
 {
	const size_t page_size = os::page_size();
	const size_t remainder = stack_size % page_size;
	if (remainder > 0) { // round up to nearest multiple
		stack_size += page_size - remainder;
	}
	const size_t real_size = stack_size + (page_size * 2);
	static const int flags = MAP_PRIVATE | MAP_ANONYMOUS | MAP_GROWSDOWN | MAP_STACK;
	void * ptr = ::mmap(nullptr, real_size, PROT_NONE, flags, -1, 0);
	if (ptr == MAP_FAILED) {
		throw xtl::system_exception(__FILE__, __LINE__);
	}
	void * base = static_cast<uint8_t *>(ptr) + page_size;
	if (::mprotect(base, stack_size, PROT_READ | PROT_WRITE) != 0) {
		const auto old_errno = errno;
		::munmap(ptr, real_size);
		throw xtl::system_exception(__FILE__, __LINE__, old_errno);
	}
	m_base = base;
	m_size = stack_size;
}

stack::~stack() noexcept {
	const size_t page_size = os::page_size();
	::munmap(static_cast<uint8_t *>(m_base) - page_size, m_size + (page_size * 2));
}

} // namespace xtl
