#include <xtl/thread.hpp>

#include <xtl/exception.hpp>
#include <cstring>
#include <pthread.h>

#if defined(DEBUG_THREAD) || defined(DEBUG_ALL)
#define fdebug(fmt, ...) fprintf(stderr, "thread[%d]: " fmt "\n", __LINE__, __VA_ARGS__);
#define debug(msg) fprintf(stderr, "thread[%d]: " msg "\n", __LINE__);
#else
#define fdebug(fmt, ...)
#define debug(msg)
#endif

namespace xtl {

class thread_impl {
public:
	inline thread_impl(xtl::function<void()> & function)
	: m_function(std::move(function))
	{
		if (::pthread_create(&m_handle, nullptr, threadfunc, this) != 0) {
			throw xtl::system_exception(__FILE__, __LINE__);
		}
		fdebug("Created new thread %p", (void *) this);
	}

	inline ~thread_impl() noexcept {
		::pthread_join(m_handle, nullptr);
		fdebug("Destroyed thread %p", (void *) this);
	}

private:
	xtl::function<void()> m_function;
	pthread_t m_handle;

	static void * threadfunc(void * ptr) {
		try {
			static_cast<thread_impl *>(ptr)->m_function();
		} catch (const std::exception & e) {
			fprintf(stderr, "%s\n", e.what());
		}
		return nullptr;
	}
};

thread::thread(xtl::function<void()> function)
: shared_ptr(std::make_shared<xtl::thread_impl>(function))
{}

} // namespace xtl
